const express = require("express");
const app = express();
const port = 2428;
const server = app.listen(port, () => console.log(`ARNweb API running on port ${port}`));

const path = require("path");
const style = require("ansi-styles");
const Gamedig = require("gamedig");

const io = require("socket.io")(server, {
    path: "/socket.io",
    cors: {
        origin: true
    }
});

var ns = {
    chat: io.of("/chat"),
    tanks: io.of("/tanks")
};

// Allow CORS from ARNweb.nl
app.use(function (req, res, next) {
    res.header("Access-Control-Allow-Origin", "https://arnweb.nl");
    res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
    next();
});

// Request documentation
app.get("/", (req, res) =>
    res.status(200).sendFile(path.resolve(__dirname + "/../html/index.html"))
);

app.get("/css/:stylesheet", function (req, res) {
    res.status(200).sendFile(path.resolve(__dirname + "/../css/" + req.params.stylesheet));
});

app.get("/res/:icon", function (req, res) {
    res.status(200).sendFile(path.resolve(__dirname + "/../res/" + req.params.icon));
});

// Socket.io chat server side
const chat = require("./services/chat.js");
ns.chat.on("connection", function (socket) {
    chat.handle(ns.chat, socket);
});

// Chat socket and info
app.get("/chat/", function (req, res) {
    // Server browser
    res.status(200).json({
        "status": "online",
        "query": {
            "path": req.path,
            "time": Date.now()
        },
        "users": chat.users
    });
});

// Socket.io tanks server side
const tanks = require("./services/tanks.js");
ns.tanks.on("connection", function (socket) {
    tanks.handle(ns.tanks, socket);
});

app.get("/tanks", function (req, res) {
    res.status(200).json({
        "status": "online",
        "query": {
            "path": req.path,
            "time": Date.now()
        },
        "players": tanks.players
    });
});

// System info
const system = require("./services/system.js");
app.get("/system/", function (req, res) {
    system.handle(req, res);
});

// cURL
app.get("/curl/", function (req, res) {
    const userAgent = req.headers["user-agent"];
    if (userAgent.search(/curl|wget/i) !== -1) {
        res.status(200).send(` 
            ${style.green.open}Hello Green!${style.green.close}
            ${style.red.open}Hello Red!${style.red.close}
        `);
    } else {
        res.status(400).json({
            "error": "Error: Only cURL requests."
        });
    }

});

// Game servers
app.get("/:type/:host", function (req, res) {
    var query = {
        type: req.params.type
    };

    var host = String(req.params.host).split(":");
    if (host.length > 1) {
        query.host = host[0],
        query.port = host[1];
    } else {
        query.host = req.params.host;
    }

    Gamedig.query(
        query
    ).then((state) => {
        res.status(200).json(state);
    }).catch((error) => {
        query.path = req.path;
        query.time = Date.now();
        res.status(503).json({
            "error": error.message,
            "query": query
        });
    });
});