var striptags = require("striptags");
var MarkdownIt = require("markdown-it"),
    md = new MarkdownIt();
var users = [];

// Function for easy server broadcasts
function serverMsg(content) {
    return {
        "username": "Server",
        "mailHash": "470965550d3f019830c421f0535d9268",
        "message": content
    };
}

function handle(io, socket) {
    // Send identification request to client
    socket.emit("user_identify");

    // Bind username and mailhash to client
    socket.on("user_login", function (user) {
        socket.username = user.username;
        socket.mailHash = user.mailHash;
        if (users.length > 0) {
            socket.emit("message_receive", serverMsg(`Hello ${socket.username}, welcome to ARNweb Chat. The following people are currently online: ${users.join(", ")}`));
        } else {
            socket.emit("message_receive", serverMsg(`Hello ${socket.username}, welcome to ARNweb Chat. You are the only one here at the moment`));
        }

        io.emit("message_receive", serverMsg(`${socket.username} has connected`));
        users.push(socket.username);
    });

    // Send disconnect broadcast to all clients
    socket.on("disconnect", function () {
        io.emit("message_receive", serverMsg(`${socket.username} has disconnected`));
        users.splice(users.indexOf(socket.username), 1);
    });

    // Handle messages
    socket.on("message_send", function (message) {
        if (socket.username) {
            // Build message data for clients
            message = striptags(message, "<br />");
            if (message !== "") {
                message = md.render(message);
                message = message.replace(/\r?\n/g, "<br />");
                var msg = {
                    "username": socket.username,
                    "mailHash": socket.mailHash,
                    "message": message
                };
                // Send message to clients
                io.emit("message_receive", msg);
            } else {
                socket.emit("message_receive", serverMsg("Empty messages will not be sent."));
            }
        }
    });
}

module.exports = {
    handle,
    users
};